/*--------------------------------------------------------------------
 *
 *  Main Include file
 *  This file contains all the macros and defines needed by the driver
 *  and user programs using it.
 *
 *--------------------------------------------------------------------*/

#ifndef _AB_H
#define _AB_H

/*-- Defines & types -------------------------------------------------*/

#define AB_NAME			"ab"		/* driver name */
#define AB_DEV_NAME		"/dev/ab"	/* device node */

#define AB_MAX_DEVICES          4       /* maximum number of boards/devices */

#define AB_LOCK_P2AFIFO		0	/* bit numbers used for locking */
#define AB_LOCK_A2PFIFO		1

#define AB_FLAG_A2PDONE		16	/* these are set by IRQ handler */
#define AB_FLAG_P2ADONE		17	/* on transfer complete */

#define AB_WAIT_P2A		1	/* wait flags */
#define AB_WAIT_A2P		2

//#define AB_OPREGS_OFFSET	0		/* PCI opregs offset for mmap() */
//#define AB_PT1_OFFSET		(1 * PAGE_SIZE)	/* pass-thru 1 offset for mmap() */
//#define AB_DRV_OFFSET		(2 * PAGE_SIZE)	/* driver status offset for mmap() */
//#define AB_BUF_OFFSET		(3 * PAGE_SIZE)	/* buffer offset for mmap() */

typedef unsigned long long	u_llong;	/* for the rdtsc op */
typedef long long		llong;

/*-- Data structures -------------------------------------------------*/

/*
 *  struct ab_driver
 *  Holds driver status information available to applications.
 *  It is physically located in a shared memory page.
 */

struct ab_driver {
    caddr_t	pd_virt;	/* buffer virtual address */
    caddr_t	pd_phys;	/* buffer physical address */
    caddr_t	pd_virt_read;	/* buffer virtual address for mem to card DMA */
    caddr_t	pd_phys_read;	/* buffer physical address for mem to card DMA */
    u_int drv_lock;	/* device locking */
    __u32 drv_icsr;	/* copy of ICSR on interrupt */
    u_llong drv_time;	/* last interrupt time */
    __u32 buffer_size;
    __u32 counter;
	__u32 irq_flag;
	__u32 buffer_number;
};

/*-- Ioctls ----------------------------------------------------------*/

#define AB_IOCTL_BASE		0x00
#define AB_FIFO_IOCTL_BASE	0x40
#define AB_MB_IOCTL_BASE	0x80
#define AB_PT_IOCTL_BASE	0xC0

/* general (from 0x00) */

#define ABIORESET			_IO('z', (AB_IOCTL_BASE + 0))
#define ABIORESETADDON		_IO('z', (AB_IOCTL_BASE + 1))
#define ABIOSETLATENCY		_IOW('z', (AB_IOCTL_BASE + 2), u_char)
#define ABIOSTART			_IOW('z', (AB_IOCTL_BASE + 3), u_int)
#define ABIOSTOP			_IO('z', (AB_IOCTL_BASE + 4))
#define ABIOCONFIGURE		_IOW('z', (AB_IOCTL_BASE + 5), u_int)
#define ABIOGMCSR			_IOR('z', (AB_IOCTL_BASE + 6), u_int)
#define ABIOGINTCSR			_IOR('z', (AB_IOCTL_BASE + 7), u_int)
#define ABIOGINFOADDR		_IOR('z', (AB_IOCTL_BASE + 8), u_int)
#define ABIOGMWTC			_IOR('z', (AB_IOCTL_BASE + 9), u_int)
#define ABIOGMRTC			_IOR('z', (AB_IOCTL_BASE + 10), u_int)
#define ABIOGCOUNTER		_IOR('z', (AB_IOCTL_BASE + 11), u_int)
#define ABIOGCPUTIME		_IOR('z', (AB_IOCTL_BASE + 12), u_int)
#define ABIOGBUFFERSIZE		_IOR('z', (AB_IOCTL_BASE + 13), u_int)
#define ABIOGA2PBUFFER		_IOR('z', (AB_IOCTL_BASE + 14), u_int)
#define ABIOGP2ABUFFER		_IOR('z', (AB_IOCTL_BASE + 15), u_int)
#define ABIOGBUFFNUMBER		_IOR('z', (AB_IOCTL_BASE + 16), u_int)
#define ABIOGNUMBUFFERS		_IOR('z', (AB_IOCTL_BASE + 17), u_int)
#define ABIOSETNUMBUFFERS	_IOW('z', (AB_IOCTL_BASE + 18), u_int)
#define ABIOSETBUFFERSIZE	_IOW('z', (AB_IOCTL_BASE + 19), u_int)

/* FIFO related (from 0x40) */

//#define ABIOWAITDMA		_IOW('z', (AB_FIFO_IOCTL_BASE + 0), u_int)

/* Pass-thru related (from 0xC0) */

//#define ABIOGETPTSIZE          _IOR('z', (AB_PT_IOCTL_BASE + 0), size_t)

#include <linux/pci.h>

/*--------------------------------------------------------------------*
 * 
 *  KERNEL ONLY Section
 *
 *--------------------------------------------------------------------*/

#ifdef __KERNEL__

/*-- Kernel compatibility --------------------------------------------*/

#ifndef LINUX_VERSION_CODE
# include <linux/version.h>
#endif

#if (LINUX_VERSION_CODE < 0x020100)
# include <linux/mm.h>
# define copy_from_user	memcpy_fromfs
# define copy_to_user	memcpy_tofs
# define ioremap	vremap
# define iounmap	vfree
#else
# include <asm/uaccess.h>
#endif

/*-- Support macros & functions --------------------------------------*/

extern inline u_llong
getclock (void)
{
    u_llong tsc;

    __asm__  __volatile__ ("rdtsc" \
			   : "=A" (tsc) \
			   : /* no input*/ );

    return tsc;
}

/*-- Defines ---------------------------------------------------------*/

#define AB_MAJOR	120			/* major number */
#define AB_PT_NUM	PCI_BASE_ADDRESS_1	/* pass-thru area */

/*-- Structures ------------------------------------------------------*/

/*
 *  struct autobahn.
 *  This structure holds global data for a single PCI board.
 */

struct autobahn {
    struct ab_driver *ab_drv;	/* shared driver status info (logical) */
    caddr_t	ab_drv_p;	/* shared driver status info (physical) */
//    struct s5933_opreg *ab_opregs;	/* PCI operation registers (logical) */
    caddr_t	ab_opregs_p;		/* PCI operation registers (physical) */
//    char       *ab_ptaddr;	/* pass-thru logical address */
//    caddr_t	ab_ptaddr_p;	/* pass-thru physical address */
//    size_t	ab_ptsize;	/* pass-thru dimension */
    struct pci_dev *pdev;       /* PCI device (contains IRQ, bus, function) */
	struct fasync_struct *async_queue;
};

/*
 *  struct ab_procdata
 *  Holds per-process information
 */

struct ab_procdata {
    size_t	pd_size;	/* buffer size */
    u_int	pd_id;		/* process ID */
    int         pd_if;          /* I/F number */
	int		pd_num;
};

#endif // __KERNEL__
#endif // __AB_H
//#endif __KERNEL__
//#endif _AB_H
